""" Panoptes - Cloud Providers

Responsible to import analysis functions from cloud providers.

Every module folder must be created like the following one:
<CLOUD_PROVIDER_NAME>/<CLOUD_PROVIDER_NAME>_panoptes.py
"""

from panoptes.cloud_providers import aws


if __name__ == "__main__":
    pass
