""" Panoptes - Main Module

Responsible only to import the cloud respective folders, which were created to
organize the project properly.
"""

from panoptes import cloud_output
from panoptes import cloud_providers
from panoptes import cloud_authentication


if __name__ == "__main__":
    pass
